# rubocop: disable Metrics/ModuleLength
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent.parent.parent + 'easy_type' + 'lib')
require 'puppet/parameter/boolean'
require 'easy_type'
require 'puppet_x/enterprisemodules/settings'
require 'puppet_x/enterprisemodules/ldap_access'
require 'puppet_x/enterprisemodules/ldap'

# docs
module Puppet
  #
  Type.newtype(:ldap_principal) do
    include EasyType
    include PuppetX::EnterpriseModules::Settings
    include PuppetX::EnterpriseModules::LdapAccess

    def self.module_name
      'ldap_principal'
    end

    desc <<-EOD
      This type allows you to manage a LDAP entry (e.g. a principal) on an LDAP server.

      Here is an example on how to use this:

          ldap_principal { 'ldap_server:cn=hajee,dc=example,dc=org':
            ensure             => 'present',
            attributes_added   => {
                'Group'        => 'superusers'
            },
            attributes         => {
                'userPassword' => 'MySecretPassword',
                'GivenName'    => 'Bert Hajee',
                'Password'     => 'verysecret',
            }
            transform         => {'password' => 'ssha512'}
          }


    EOD

    #
    # Use the generate method to delete any principal that is not defined in
    # the manifest.
    #
    def generate
      return unless purge
      non_defined_principals.collect do |principal|
        Puppet.info "Removing #{principal} because purge is set to true in container."
        self.class.new(:name => principal, :ensure => 'absent')
      end
    end

    def non_defined_principals
      available_principals - defined_principals
    end

    def defined_principals
      catalog.resources.select { |r| r.class == self.class }.map(&:name)
    end

    def available_principals
      ldap(ldap_server).search(:base => dn).collect { |e| "#{ldap_server}:#{e.dn}" }
    end

    def ldap_server
      name.split(':')[0]
    end

    def dn
      name.split(':')[1]
    end

    def parent_dn
      entries = dn.split(',')
      entries.shift
      entries.join(',')
    end

    def unspecfied_attributes
      provider.attributes.keys - attributes.keys
    end

    def unspecfied_attributes?
      unspecfied_attributes.any?
    end

    to_get_raw_resources do
      configuration.keys.collect do |ldap_server|
        Puppet.info "index #{ldap_server}"
        results = ldap(ldap_server).search({})
        results.each { |e| e['ldap_server'] = ldap_server }
        results
      end.flatten
    end

    on_prefetch do |resources, provider|
      resources.each do |name, _value|
        ldap_server, dn = name.split(':')
        entries = ldap(ldap_server).search(:base => dn)
        entry = entries.first # If multiple entries exists, take the first one
        next unless entry
        entry['ldap_server'] = ldap_server
        value = provider.map_raw_to_resource(entry)
        resources[name].provider = value if value.name == name
      end
    end

    on_create do
      Puppet.info "create #{name} "
      all_attributes = self[:attributes].merge(self[:present_in])
      ldap.add(distinguished_name, all_attributes)
      nil
    end

    on_modify do
      Puppet.info "modify #{name}"
      if purge_attributes && unspecfied_attributes?
        unspecfied_attributes.each do |attribute|
          ldap.delete_attribute(distinguished_name, attribute)
        end
      end
      # Do nothing. Leave it to the attributes property
      nil
    end

    on_destroy do
      Puppet.info "destroy #{name} "
      if force
        ldap.delete_tree(distinguished_name)
      else
        ldap.delete(distinguished_name)
      end
      nil
    end

    ensurable
    parameter :ldap_server
    parameter :distinguished_name
    parameter :name
    parameter :purge
    parameter :purge_attributes
    parameter :force
    parameter :transform

    property  :attributes
    property  :present_in
    property  :absent_in

    map_titles_to_attributes([/^((.+):(.+))$/, [:name, :ldap_server, :distinguished_name]])

    autobefore(:ldap_principal) { "#{ldap_server}:#{parent_dn}" if self[:ensure] == :absent }
    autorequire(:ldap_principal) { "#{ldap_server}:#{parent_dn}" if self[:ensure] == :present }
    autorequire(:ldap_server) { ldap_server }
  end
end
