require 'puppet_x/enterprisemodules/transformers'
require 'puppet_x/enterprisemodules/attribute_helpers'

newproperty(:attributes) do
  include EasyType
  include PuppetX::EnterpriseModules::LdapAccess
  include PuppetX::EnterpriseModules::Transformers
  include PuppetX::EnterpriseModules::AttributeHelpers

  defaultto({})

  desc <<-EOD
    Set's the attributes of the specfied LDAP entry.

    Using the property, you can set the attributes of a LDAP entry. The ldap_principal has
    who modes of operation, see the `purge_attributes` for more details about the two modes.

    When the `purge_attributes` is not specfied or false, Puppet **only** works on th attributes specfied.
    All other attributes will be untouched. Here is an example:

        ldap_principal { 'docker:cn=piggy,ou=muppetshow,dc=example,dc=org':
          ...
          attributes => {
            'sn'           => 'piggy',
            'objectclass'  => ['top', 'inetOrgPerson'],
            'userPassword' => "{SSHA512}2ZsyGTxVyEw14Cu9D/OXpTddfy/387D/rlR6R0VVdRIz+3Wn52fSYZpKAP1S\n9J/kRbkoBiPK/9eZMOZV6cgidzEyMzQ1Njc4",
            'givenName'    => 'Miss Piggy',
          },
          ...
        }

  EOD

  data_type("Optional[Hash]")

  to_translate_to_resource do |raw_resource|
    valid_attributes = raw_resource.attribute_names.reject { |e| [:dn, :ldap_server].include?(e) }
    values = valid_attributes.collect do |attribute|
      value = raw_resource.send(attribute)
      if value.size == 1
        [attribute, value.first]
      else
        [attribute, value]
      end
    end
    ::Hash[values]
  end

  def insync?(is)
    return false if unspecified_attributes? && resource.purge_attributes
    merged_should = merge_change_only_attributes(should)
    updated_attributes(is, merged_should).empty?
  end

  on_modify do
    operations = []
    updated_attributes(is, should).each do |attribute|
      operations << [:replace, attribute, should[attribute]]
    end
    ldap(resource[:ldap_server]).modify(resource[:distinguished_name], operations) if operations.any?
    nil
  end

  def change_to_s(is, should)
    message = []
    updated_attributes(is, should).each do |attribute|
      message << if should_be_transformed(attribute)
                   change_message(attribute, '[current transformed value]', '[new transformed value]')
                 else
                   change_message(attribute, is[attribute], should[attribute])
                 end
    end
    message << "clearing attribute(s) #{unspecified_attributes.join(', ')} because purge_attributes is true" if resource.purge_attributes && unspecified_attributes?
    message.join(',')
  end

  #
  # This method merges the absent_in and the present_in properties
  # of puppet into a comparable attributes hash
  #
  def merge_change_only_attributes(values)
    out = values.dup
    resource.present_in.each { |k,v| out[k] = (Array(out[k]) << Array(v)).flatten.uniq }
    resource.absent_in.each do |k,v|
      Array(v).each do |entry|
        fail "Value #{entry} in 'absent_in' property conflicts with either 'attributes' or 'present_in' property." if out[k].include?(entry)
      end
    end
    out
  end

  def change_message(attribute, from, to)
    "changing attribute #{attribute} value from '#{from}' to '#{to}'"
  end

  def unspecified_attributes?
    unspecified_attributes.any?
  end

  def unspecified_attributes
    (is.keys - should.keys)
  end
end
